@@ -4,17 +4,57 @@
 import sys
 import os
 import unittest
-try:
-    # For Pythons w/distutils pybsddb
-    import bsddb3 as bsddb
-except ImportError:
-    # For Python 2.3
-    import bsddb
+import bsddb3 as bsddb
 
 
 if sys.version_info[0] >= 3 :
     charset = "iso8859-1"  # Full 8 bit
 
+    class logcursor_py3k(object) :
+        def __init__(self, env) :
+            self._logcursor = env.log_cursor()
+
+        def __getattr__(self, v) :
+            return getattr(self._logcursor, v)
+
+        def __next__(self) :
+            v = getattr(self._logcursor, "next")()
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
+        next = __next__
+
+        def first(self) :
+            v = self._logcursor.first()
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
+        def last(self) :
+            v = self._logcursor.last()
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
+        def prev(self) :
+            v = self._logcursor.prev()
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
+        def current(self) :
+            v = self._logcursor.current()
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
+        def set(self, lsn) :
+            v = self._logcursor.set(lsn)
+            if v is not None :
+                v = (v[0], v[1].decode(charset))
+            return v
+
     class cursor_py3k(object) :
         def __init__(self, db, *args, **kwargs) :
             self._dbcursor = db.cursor(*args, **kwargs)
@@ -23,14 +63,15 @@ if sys.version_info[0] >= 3 :
             return getattr(self._dbcursor, v)
 
         def _fix(self, v) :
-            if v == None : return None
+            if v is None : return None
             key, value = v
             if isinstance(key, bytes) :
                 key = key.decode(charset)
             return (key, value.decode(charset))
 
-        def __next__(self) :
-            v = getattr(self._dbcursor, "next")()
+        def __next__(self, flags=0, dlen=-1, doff=-1) :
+            v = getattr(self._dbcursor, "next")(flags=flags, dlen=dlen,
+                    doff=doff)
             return self._fix(v)
 
         next = __next__
@@ -71,26 +112,26 @@ if sys.version_info[0] >= 3 :
             v = self._dbcursor.next_nodup()
             return self._fix(v)
 
-        def put(self, key, value, flags=0, dlen=-1, doff=-1) :
+        def put(self, key, data, flags=0, dlen=-1, doff=-1) :
             if isinstance(key, str) :
                 key = bytes(key, charset)
-            if isinstance(value, str) :
-                value = bytes(value, charset)
-            return self._dbcursor.put(key, value, flags=flags, dlen=dlen,
+            if isinstance(data, str) :
+                value = bytes(data, charset)
+            return self._dbcursor.put(key, data, flags=flags, dlen=dlen,
                     doff=doff)
 
         def current(self, flags=0, dlen=-1, doff=-1) :
             v = self._dbcursor.current(flags=flags, dlen=dlen, doff=doff)
             return self._fix(v)
 
-        def first(self) :
-            v = self._dbcursor.first()
+        def first(self, flags=0, dlen=-1, doff=-1) :
+            v = self._dbcursor.first(flags=flags, dlen=dlen, doff=doff)
             return self._fix(v)
 
         def pget(self, key=None, data=None, flags=0) :
             # Incorrect because key can be a bare number,
             # but enough to pass testsuite
-            if isinstance(key, int) and (data==None) and (flags==0) :
+            if isinstance(key, int) and (data is None) and (flags == 0) :
                 flags = key
                 key = None
             if isinstance(key, str) :
@@ -101,7 +142,7 @@ if sys.version_info[0] >= 3 :
             if isinstance(data, str) :
                 data = bytes(data, charset)
             v=self._dbcursor.pget(key=key, data=data, flags=flags)
-            if v != None :
+            if v is not None :
                 v1, v2, v3 = v
                 if isinstance(v1, bytes) :
                     v1 = v1.decode(charset)
@@ -114,7 +155,7 @@ if sys.version_info[0] >= 3 :
 
         def join_item(self) :
             v = self._dbcursor.join_item()
-            if v != None :
+            if v is not None :
                 v = v.decode(charset)
             return v
 
@@ -134,7 +175,7 @@ if sys.version_info[0] >= 3 :
                 args =(k, d, f)
 
             v = self._dbcursor.get(*args, **kwargs)
-            if v != None :
+            if v is not None :
                 k, v = v
                 if isinstance(k, bytes) :
                     k = k.decode(charset)
@@ -176,7 +217,7 @@ if sys.version_info[0] >= 3 :
             if isinstance(k, str) :
                 k = bytes(k, charset)
             v = self._db[k]
-            if v != None :
+            if v is not None :
                 v = v.decode(charset)
             return v
 
@@ -203,12 +244,26 @@ if sys.version_info[0] >= 3 :
                 k = bytes(k, charset)
             return self._db.has_key(k, txn=txn)
 
-        def put(self, key, value, txn=None, flags=0, dlen=-1, doff=-1) :
+        def set_re_delim(self, c) :
+            if isinstance(c, str) :  # We can use a numeric value byte too
+                c = bytes(c, charset)
+            return self._db.set_re_delim(c)
+
+        def set_re_pad(self, c) :
+            if isinstance(c, str) :  # We can use a numeric value byte too
+                c = bytes(c, charset)
+            return self._db.set_re_pad(c)
+
+        def get_re_source(self) :
+            source = self._db.get_re_source()
+            return source.decode(charset)
+
+        def put(self, key, data, txn=None, flags=0, dlen=-1, doff=-1) :
             if isinstance(key, str) :
                 key = bytes(key, charset)
-            if isinstance(value, str) :
-                value = bytes(value, charset)
-            return self._db.put(key, value, flags=flags, txn=txn, dlen=dlen,
+            if isinstance(data, str) :
+                value = bytes(data, charset)
+            return self._db.put(key, data, flags=flags, txn=txn, dlen=dlen,
                     doff=doff)
 
         def append(self, value, txn=None) :
@@ -221,6 +276,11 @@ if sys.version_info[0] >= 3 :
                 key = bytes(key, charset)
             return self._db.get_size(key)
 
+        def exists(self, key, *args, **kwargs) :
+            if isinstance(key, str) :
+                key = bytes(key, charset)
+            return self._db.exists(key, *args, **kwargs)
+
         def get(self, key, default="MagicCookie", txn=None, flags=0, dlen=-1, doff=-1) :
             if isinstance(key, str) :
                 key = bytes(key, charset)
@@ -230,7 +290,7 @@ if sys.version_info[0] >= 3 :
             else :
                 v=self._db.get(key, txn=txn, flags=flags,
                         dlen=dlen, doff=doff)
-            if (v != None) and isinstance(v, bytes) :
+            if (v is not None) and isinstance(v, bytes) :
                 v = v.decode(charset)
             return v
 
@@ -238,7 +298,7 @@ if sys.version_info[0] >= 3 :
             if isinstance(key, str) :
                 key = bytes(key, charset)
             v=self._db.pget(key, txn=txn)
-            if v != None :
+            if v is not None :
                 v1, v2 = v
                 if isinstance(v1, bytes) :
                     v1 = v1.decode(charset)
@@ -252,7 +312,7 @@ if sys.version_info[0] >= 3 :
             if isinstance(value, str) :
                 value = bytes(value, charset)
             v=self._db.get_both(key, value, txn=txn, flags=flags)
-            if v != None :
+            if v is not None :
                 v = v.decode(charset)
             return v
 
@@ -288,18 +348,21 @@ if sys.version_info[0] >= 3 :
                         key = key.decode(charset)
                     data = data.decode(charset)
                     key = self._callback(key, data)
-                    if (key != bsddb._db.DB_DONOTINDEX) and isinstance(key,
-                            str) :
-                        key = bytes(key, charset)
+                    if (key != bsddb._db.DB_DONOTINDEX) :
+                        if isinstance(key, str) :
+                            key = bytes(key, charset)
+                        elif isinstance(key, list) :
+                            key2 = []
+                            for i in key :
+                                if isinstance(i, str) :
+                                    i = bytes(i, charset)
+                                key2.append(i)
+                            key = key2
                     return key
 
-            if bsddb.db.version() < (4, 1) :
-                return self._db.associate(secondarydb._db,
-                        associate_callback(callback).callback, flags=flags)
-            else :
-                return self._db.associate(secondarydb._db,
-                        associate_callback(callback).callback, flags=flags,
-                        txn=txn)
+            return self._db.associate(secondarydb._db,
+                    associate_callback(callback).callback, flags=flags,
+                    txn=txn)
 
         def cursor(self, txn=None, flags=0) :
             return cursor_py3k(self._db, txn=txn, flags=flags)
@@ -315,6 +378,19 @@ if sys.version_info[0] >= 3 :
         def __getattr__(self, v) :
             return getattr(self._dbenv, v)
 
+        def log_cursor(self, flags=0) :
+            return logcursor_py3k(self._dbenv)
+
+        def get_lg_dir(self) :
+            return self._dbenv.get_lg_dir().decode(charset)
+
+        def get_tmp_dir(self) :
+            return self._dbenv.get_tmp_dir().decode(charset)
+
+        def get_data_dirs(self) :
+            return tuple(
+                (i.decode(charset) for i in self._dbenv.get_data_dirs()))
+
     class DBSequence_py3k(object) :
         def __init__(self, db, *args, **kwargs) :
             self._db=db
@@ -358,14 +434,8 @@ if sys.version_info[0] >= 3 :
     do_proxy_db_py3k.flag = False
     do_proxy_db_py3k(True)
 
-try:
-    # For Pythons w/distutils pybsddb
-    from bsddb3 import db, dbtables, dbutils, dbshelve, \
-            hashopen, btopen, rnopen, dbobj
-except ImportError:
-    # For Python 2.3
-    from bsddb import db, dbtables, dbutils, dbshelve, \
-            hashopen, btopen, rnopen, dbobj
+from bsddb3 import db, dbtables, dbutils, dbshelve, \
+        hashopen, btopen, rnopen, dbobj
 
 try:
     from bsddb3 import test_support
@@ -400,27 +470,23 @@ if 'silent' in sys.argv:  # take care of old flag, just in case
 def print_versions():
     print()
     print('-=' * 38)
-
-    if sys.version_info[0] < 3 :
-        print(db.DB_VERSION_STRING)
-    else :
-        print(db.DB_VERSION_STRING.decode(charset))
-
+    print(db.DB_VERSION_STRING)
     print('bsddb.db.version():   %s' % (db.version(), ))
+    if db.version() >= (5, 0) :
+        print('bsddb.db.full_version(): %s' %repr(db.full_version()))
+    print('bsddb.db.__version__: %s' % db.__version__)
+    print('bsddb.db.cvsid:       %s' % db.cvsid)
 
-    if sys.version_info[0] < 3 : 
-        print('bsddb.db.__version__: %s' % db.__version__)
-    else :
-        print('bsddb.db.__version__: %s' % db.__version__.decode(charset))
+    # Workaround for allowing generating an EGGs as a ZIP files.
+    suffix="__"
+    print('py module:            %s' % getattr(bsddb, "__file"+suffix))
+    print('extension module:     %s' % getattr(bsddb, "__file"+suffix))
 
-    if sys.version_info[0] < 3 :
-        print('bsddb.db.cvsid:       %s' % db.cvsid)
-    else :
-        print('bsddb.db.cvsid:       %s' % db.cvsid.decode(charset))
-
-    print('py module:            %s' % bsddb.__file__)
-    print('extension module:     %s' % bsddb._bsddb.__file__)
-    print('python version:       %s' % sys.version)
+    print('Test working dir:     %s' % get_test_path_prefix())
+    import platform
+    print('python version:       %s %s' % \
+            (sys.version.replace("\r", "").replace("\n", ""), \
+            platform.architecture()[0]))
     print('My pid:               %s' % os.getpid())
     print('-=' * 38)
 
@@ -456,7 +522,8 @@ def get_new_database_path() :
 
 # This path can be overriden via "set_test_path_prefix()".
 import os, os.path
-get_new_path.prefix=os.path.join(os.sep,"tmp","z-Berkeley_DB")
+get_new_path.prefix=os.path.join(os.environ.get("TMPDIR",
+    os.path.join(os.sep,"tmp")), "z-Berkeley_DB")
 get_new_path.num=0
 
 def get_test_path_prefix() :
@@ -503,6 +570,8 @@ def suite(module_prefix='', timing_check=None):
     test_modules = [
         'test_associate',
         'test_basics',
+        'test_dbenv',
+        'test_db',
         'test_compare',
         'test_compat',
         'test_cursor_pget_bug',
@@ -511,6 +580,7 @@ def suite(module_prefix='', timing_check=None):
         'test_dbtables',
         'test_distributed_transactions',
         'test_early_close',
+        'test_fileid',
         'test_get_none',
         'test_join',
         'test_lock',
