@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 
 import sys, os
-refactor_path="/usr/local/lib/python3.0/"
+refactor_path="/usr/local/lib/python3.3/"
 
 def make2to3(path_from, path_to) :
     if os.path.isdir(path_from) :
         if path_from.endswith(".svn") : return
+        if path_from.endswith(".hg") : return
         try :
             os.mkdir(path_to)
         except :
@@ -15,8 +16,8 @@ def make2to3(path_from, path_to) :
         return
 
     cwd = os.getcwd()
-    if os.path.exists(path_to) and \
-        (os.stat(path_from).st_ctime < os.stat(path_to).st_ctime) :
+    if (not path_from.endswith(".py")) or (os.path.exists(path_to) and \
+        (os.stat(path_from).st_mtime < os.stat(path_to).st_mtime)) :
             return
     print "*** Converting", path_to
     if path_from[0] != "/" :
@@ -45,6 +46,8 @@ def make2to3(path_from, path_to) :
 
     return bool(retcode)
 
+print "Using '%s' for 2to3 conversion tool" %refactor_path
+
 make2to3("setup2.py", "setup3.py")
 make2to3("test2.py", "test3.py")
 make2to3("Lib", "Lib3")
